<?php

namespace HCM\AVL\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Zeedhi\Framework\DTO;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Response;
use HCM\AVL\Logic\AvaliacaoPesquisa;

class CalculoApuracaoAvaliacaoService {
    
    protected $entityManager;
    protected $hcmParameters;
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function inicarApuracaoCompleta($nravaliacao, DTO\Response $response){
        
        $avaliacaoPesquisaLogic = new AvaliacaoPesquisa($this->entityManager, $this->hcmParameters);
        $messages = array();
        $resultParticipOk = $avaliacaoPesquisaLogic->retornaColaboradoresAvaliacaoConcluidaLogic($nravaliacao, $messages);
        $resultFaixaCrit  = $avaliacaoPesquisaLogic->retornaFaixaCriteriosLogic($nravaliacao);
        if (!$resultFaixaCrit) {
            $response->addNotification(new Notification(Messages::HCM03000_AVL_CRITERIO_RESULTADO_NAO_DEFINIDO, Notification::TYPE_ALERT));
        }
        //Inicia Apuração
        $totResultApur = $avaliacaoPesquisaLogic->iniciaApuracaoResultadosLogic($resultParticipOk, $nravaliacao, $resultFaixaCrit, $messages);
        
        //Salva Mensagens
        $response = $this->preparaMensagens($messages, $response);
        
        return $totResultApur;
    }
    
    public function iniciarApuracaoParcial($row, DTO\Response $response){

        $avaliadosSelecionados = $row['AVALIADO'];
        $messages = array();
        $nravaliacao = $row['NRAVALIACAO'];
        $nrorg  =$row['NRORG'];
        $count = 0;
        $agrupamento = false;
        
        foreach ($avaliadosSelecionados as $avaliado) {

            $avaliacaoPesquisaLogic = new AvaliacaoPesquisa($this->entityManager, $this->hcmParameters);

            //retorna a faixa de criterios da avaliacao informada
            $resultFaixaCrit = $avaliacaoPesquisaLogic->retornaFaixaCriteriosLogic($nravaliacao);
            if (!$resultFaixaCrit) {
                $response->addNotification(new Notification(Messages::HCM03000_AVL_CRITERIO_RESULTADO_NAO_DEFINIDO, Notification::TYPE_ALERT));
            }
            
            $nrparticipaval = $avaliado['NRPARTICIPAVAL'];
            /* @var $participaval \Avaliacao\Entities\Participaval */
            $participaval   = $this->entityManager->getRepository(Repositories::PARTICIPAVAL)->findOneBy(array('nravaliacao' => $nravaliacao, 'nrparticipaval' => $nrparticipaval, 'nrorg' => $nrorg));
            $itemApuresult  = $this->entityManager->getRepository(Repositories::ITEMAPURESULT)->findOneBy(array( 'nrparticipaval' => $nrparticipaval, 'nrorg' => $nrorg));
            if(empty($itemApuresult)){
                $combQuest       = $participaval->getNrcombquestaval();
                $nrparticipaval  = $avaliado['NRPARTICIPAVAL'];
                $parcAval        = $avaliado['NRPARCAVALIADO'];
                $nrparcexecaval  = $avaliado['NRPARCEXECAVAL'];
                $nmParcAval      = $avaliado['AVALIADO'];
                $nmParcAvaliador = $avaliado['AVALIADOR'];
                $apuracao = $avaliacaoPesquisaLogic->apurarResultadoColaborador($combQuest, $parcAval, $nrparcexecaval, $nravaliacao, $resultFaixaCrit, $nrparticipaval, $messages);
                $agrupamento = $apuracao['agrup'];    
                
            }else{
                $count ++;
            }
            //Inicia Apuração
        }
        if($count > 0){
        array_push($messages, $count." colaborador(es) já apurado(s)");
        }
        if($agrupamento === true){
            array_push($messages, "Avaliação '$nravaliacao' não possui agrupamento.");
        }
        $response = $this->preparaMensagens($messages, $response);
        return (count($avaliadosSelecionados)-$count);
    }
    
    public function iniciarRetrocessoCompleto($nravaliacao, DTO\Response $response){
        
        $messages = array();
        $avaliacaoPesquisaLogic = new AvaliacaoPesquisa($this->entityManager, $this->hcmParameters);
        $totResultRet = $avaliacaoPesquisaLogic->desfazApuracaoLogic($nravaliacao);
        $qtColabs = $totResultRet['colaborador'] - $totResultRet['item'];
        if($totResultRet['agrup'] > 0){
            array_push($messages, "Avaliação ".$nravaliacao." não possui agrupamento(s)");
        }
        
        if($totResultRet['item'] > 0){
            if($qtColabs > 0){
                array_push($messages, $totResultApur['item']." colaborador(es) não apurado(s)");
                $response = $this->preparaMensagens($messages, $response);
                return ($qtColabs);
            }else{
                array_push($messages, "Colaborador(es) não apurado(s)");
                $response = $this->preparaMensagens($messages, $response);
                return 0;
            }
        }else {
            return ($totResultRet['colaborador']);
        }
        $response = $this->preparaMensagens($messages, $response);
        
    }
    
    public function iniciarRetrocessoParcial($row, DTO\Response $response){
        
        $avaliadosSelecionados = $row['AVALIADO'];
        $nravaliacao = $row['NRAVALIACAO'];
        $nrorg  =$row['NRORG'];
        $countItem = 0;
        $agrupItem = 0;
        $messages = array();
        
        foreach ($avaliadosSelecionados as $avaliado) {

            $parcAval       = $avaliado['NRPARCEXECAVAL'];
            $nrparticipaval = $avaliado['NRPARTICIPAVAL'];

            /* @var $participaval \Avaliacao\Entities\Participaval */
            $participaval   = $this->entityManager->getRepository(Repositories::PARTICIPAVAL)->findOneBy(array('nravaliacao' => $nravaliacao, 'nrparticipaval' => $nrparticipaval, 'nrorg' => $nrorg));
            $nrComb = $participaval->getNrcombquestaval();

            $paramsUpdate = array('p_vrresultavalia' => null,
                                  'p_cdtiporesu'     => null,
                                  'p_nrparcavaliado' => $parcAval,
                                  'p_nrcombquestaval'=> $nrComb,
                                  'p_nravaliacao'    => $nravaliacao,
                                  'p_nrorg'          => $nrorg);

            $avaliacaoPesquisaLogic = new AvaliacaoPesquisa($this->entityManager, $this->hcmParameters);
            $retrocesso = $avaliacaoPesquisaLogic->updateDesfazApuracaoLogic($paramsUpdate);
            if($retrocesso['item'] === 1){
                $countItem ++;
            }
            if($retrocesso['agrup'] === 1){
                $agrupItem ++;
            }
        }
        $qtColabs = count($avaliadosSelecionados) - $countItem;
        if($agrupItem > 0){
            array_push($messages, "Avaliação ".$nravaliacao." não possui agrupamento(s)");
        }
        
        if($countItem > 0){
            if($qtColabs > 0){
                array_push($messages, $countItem." colaborador(es) não apurado(s)");
                $response = $this->preparaMensagens($messages, $response);
                return ($qtColabs);
            }else{
                array_push($messages, "Colaborador(es) não apurado(s)");
                $response = $this->preparaMensagens($messages, $response);
                return 0;
            }
        }else {
            return (count($avaliadosSelecionados));
        }
        
        $response = $this->preparaMensagens($messages, $response);
    }
    
    
    public function preparaMensagens($messages, $response){
        foreach ($messages as $message) {
            $response->addNotification(new Notification($message, Notification::TYPE_ALERT));
        }
        return $response;
    }
}