<?php
namespace HCM\FPA\Controllers;

use HCM\FPA\Services\ContaContabilService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\FilterCriteria;


class ContaContabilController extends Crud {

    protected $dataSourceName;
    protected $parameterBag;
    protected $crudService;
    protected $dataSourceNameToSave  = false;
    protected $service;
    
    protected $parameterBagColumns = array('P_NRORG','P_NRORG_PADRAO','P_CDCONTCTBL');


    public function __construct($crudService, ParameterBag $parameterBag, ContaContabilService $service) {
        $this->service = $service;
        $this->parameterBag = $parameterBag;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function getContaContabil(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("contctbl_arvore");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getContCTBL(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("contctbl");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function deleteContaContabil(DTO\Request $request, DTO\Response $response){
        try {
            $this->dataSourceName = "contctbl_arvore";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getContaContabilSubC(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("contctblcorre_grupocontacorre_natucontcorre");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function verificaExclusao(DTO\Request $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
    		$resul = $this->service->verificaExclusao($row);
            $response->addDataSet( new \Zeedhi\Framework\DataSource\DataSet('verificaExclusao', $resul) );                    
            // $response->addDataSet(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarSubContas(DTO\Request $request, DTO\Response $response) {
        try {
    		$row   = $request->getRow();
    		$this->service->salvarSubContas($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deletarSubContas(DTO\Request $request, DTO\Response $response) {
        try {
    		$row   = $request->getRow();
    		$this->service->deletarSubContas($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    public function verificaContaNivelAnterior(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName='contctbl';
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarContaContabil (DTO\Request $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
    		    $this->service->salvarContaContabil($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarCadastramentoAutomatico (DTO\Request $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
    		    $this->service->salvarCadastramentoAutomatico($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function retornaMaiorConta(DTO\Request $request, DTO\Response $response) {
        try {
    		$row   = $request->getRow();
    		$resul = $this->service->retornaMaiorConta($row);
            $dataSet = new DataSet("contctbl", $resul);
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvaMascara(DTO\Request $request, DTO\Response $response) {
        try {
            $mascara = count($request->getDataSet()->getRows()) == 1 ? $request->getDataSet()->getRows()[0]['CDPICTCCTB'] : null;
            if(!empty($mascara)){
                $this->service->salvaMascara($mascara);
            }
            $response->addDataSet(new DataSet("MASCARA", array("CDPICTCCTB" => $mascara)));
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMascara(DTO\Request $request, DTO\Response $response) {
        try {
            $mascara = $this->service->getMascara();
            $mascara = is_array($mascara) && !empty($mascara[0]['CDPICTCCTB']) ? $mascara[0]['CDPICTCCTB'] : null;
            $response->addDataSet(new DataSet("MASCARA", array("CDPICTCCTB" => $mascara)));
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        //coloca a clausula orderBy existente no parametro $filterCriteria no novo filtro $newFilterCriteria
        foreach($filterCriteria->getOrderBy() as $key => $field){
            $newFilterCriteria->addOrderBy($key, $field);
        }

        return $newFilterCriteria;
    }

}
