<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 22/07/2016
 * Time: 16:25
 */

namespace HCM\FPA\Controllers;

use Folha\Entities\FpaTipomoviment;
use HCM\FPA\Services\ItensCalculoService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;

class ItensCalculoController extends Crud {

    protected $dataSourceName = 'fpa_itecalcfolha';
    protected $dataSourceName_get = 'getItecalcfolha';
    protected $dataSourceName_getEstrutura = 'getItecalcfolhaPorEstrutura';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTIPOMOVIMENT','P_RESCISAO','P_ULTIMODIAMES','P_NRCALCULOFOLHA', 'P_NRESTRUTLEGAL', 'P_NRESTRUTGEREN', 'NRESTRUTURA_BENEFICIO', 'NMVINCULOM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ItensCalculoService $itensCalculoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->itensCalculoService = $itensCalculoService;
        parent::__construct($dataSourceManager);
    }

    public function getItecalcfolha (DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        ini_set('memory_limit', '2048M');
        $nrtipomoviment = null;
        $nrestrutura_beneficio = null;
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        foreach($conditions as $condition) {
            if($condition['columnName'] == 'P_NRTIPOMOVIMENT') {
                $nrtipomoviment = $condition['value'];
            }
            if($condition['columnName'] == 'NRESTRUTURA_BENEFICIO') {
                $nrestrutura_beneficio = $condition['value'];
            }
            // O operador recebido do front é '=' ao invés de 'IN'
            if($condition['columnName'] == 'NRVINCULOM') {
                $condition['operator'] = 'IN';
            }
        }
        
        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );

        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        $filterData->addCondition('P_DTMESCOMPETENC','=',$this->hcmParameters->getDtCompetencia());
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        
        if($nrtipomoviment){
            $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment));
            
            if($nrtipomoviment==21){
                $filterData->addCondition('P_RESCISAO','=','T');
            }
        }
        //$filterData->addCondition('P_RESCISAO','=','N');
        $nrestrutura_beneficio ? null : $filterData->addCondition('NRESTRUTURA_BENEFICIO', '=', null);

        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $newFilterData->setPageSize(99999999999);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        /*$newDataSet = array();
        foreach($dataSet->getRows() as $data){
            $data['NRVINCULOM'] = strval($data['NRVINCULOM']);
            $data['VINCULO_GROUPED'] = $data['NMVINCULOM'].' - '.$data['NRVINCULOM'];
            array_push($newDataSet, $data);
        }*/
        $response->addDataSet($dataSet);
    }
    
    public function getItecalcfolhaPorEstrutura (DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        ini_set('memory_limit', '2048M');

        $filterData = $request->getFilterCriteria();
        foreach($filterData->getConditions() as $condition) {
            if($condition['columnName'] == 'P_NRTIPOMOVIMENT') {
                $nrtipomoviment = $condition['value'];
            }
        }
        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment));
        $filterData->setDataSourceName($this->dataSourceName_getEstrutura);
        $filterData->setPageSize(9999);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setItecalcfolha (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $msg = array();
            $disponibilidade = true;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->itensCalculoService->verificaEventoBloqueado($row['NRORG'], $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                if ($disponibilidade === true) {
                    if(array_key_exists('NRTIPOMOVIMENT1', $row)){
                        $row['NRTIPOMOVIMENT'] = $row['NRTIPOMOVIMENT1'];
                    }
                    if(array_key_exists('VRREFFOLHA_inline', $row)){
                        $row['__is_new'] = false;
                    }
                    
                    if (array_key_exists('NRCALCULOFOLHA', $row)) {
                        if ($row['NRCALCULOFOLHA'] === '') {
                            $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                        }
                    } else {
                        $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                    }
                    
                    $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                    // Somente para o vale transporte é permitido fazer alterações mesmo que já exista líquido.
                    if ($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE && $idevendemoncalc != 'S') {
                        $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);
                        if ($liquido) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Message::TYPE_ERROR));
                            return;
                        }
                    }
                    
                    if($row['__is_new']) {
                        $item = $this->itensCalculoService->retornaItemCalculo($row['NRORG'], $row['NRCALCULOFOLHA'], $row['NREVENTOM'], $row['NRVINCULOM'], $row['DTOCORRENCIA']);
                        if (is_object($item)) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_ITEM_CALC_UNIQ_ERROR), Message::TYPE_ERROR);
                            return;
                        }
                    }
    
                    if (array_key_exists('NRITEMCALCFOLHA', $row)) {
                        if ($row['NRITEMCALCFOLHA'] === '') {
                            $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                        }
                    } else {
                        $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                    }
                    
                    if(!array_key_exists('NRGOZOFERIAS', $row)){
                        $row['NRGOZOFERIAS'] = null;
                    }
                
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItecalcfolhaInline (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as &$rows) {
                foreach ($rows as &$row) {    
                    if(array_key_exists('NRTIPOMOVIMENT1', $row)){
                        $row['NRTIPOMOVIMENT'] = $row['NRTIPOMOVIMENT1'];
                    }
                    if(array_key_exists('VRREFFOLHA_inline', $row)){
                        $row['__is_new'] = false;
                    }
                    $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                    
                    if (array_key_exists('NRCALCULOFOLHA', $row)) {
                        if ($row['NRCALCULOFOLHA'] === '') {
                            $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                        }
                    } else {
                        $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                    }
                    // Somente para o vale transporte é permitido fazer alterações mesmo que já exista líquido.
                    if ($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE  && $idevendemoncalc != 'S') {
                        $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);
                        if ($liquido) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Message::TYPE_ERROR));
                            return;
                        }
                    }
                    
                    if($row['__is_new']) {
                        $item = $this->itensCalculoService->retornaItemCalculo($row['NRORG'], $row['NRCALCULOFOLHA'], $row['NREVENTOM'], $row['NRVINCULOM'], $row['DTOCORRENCIA']);
                        if (is_object($item)) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_ITEM_CALC_UNIQ_ERROR), Message::TYPE_ERROR);
                            return;
                        }
                    }
    
                    if (array_key_exists('NRITEMCALCFOLHA', $row)) {
                        if ($row['NRITEMCALCFOLHA'] === '') {
                            $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                        }
                    } else {
                        $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                    }
                    
                    $this->itensCalculoService->updateItensCalc($row);
                    
                    $message = new \Zeedhi\Framework\DTO\Response\Message('Registro salvo com sucesso', DTO\Response\Message::TYPE_MESSAGE);
                    $response->addMessage($message);
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                // Somente para o vale transporte é permitido fazer exclusão mesmo que já exista líquido.
                if ($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE && $idevendemoncalc != 'S') {
                    $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);

                    if ($liquido) {
                        $response->addNotification(new Notification(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Notification::TYPE_ERROR));
                        return;
                    }
                }
            }

            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 