<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class GrupoIncidenciaService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function deleteGrupoIncidencia($dataset) {
        
        $item = $this->entityManager->getRepository(Repositories::FPA_GRPITEM)->findBy(array("nrgrpincide" => $dataset['NRGRPINCIDE']));
        if (isset($item[0])) {
            throw new \Exception(Messages::HCM01000_FPA_GRUPINCID_EXISTS_ITEM_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $grpincidencia = $this->entityManager->getRepository(Repositories::FPA_GRPINCIDE)->findOneBy(array("nrgrpincide" => $dataset['NRGRPINCIDE'],"nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($grpincidencia);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function setGrupoIncidenciaPorArray($row){
        $fpa_grpincide = Repositories::FPA_GRPINCIDE;
        
        $grupoIncidencia = new $fpa_grpincide;
        
        $grupoIncidencia->setNrorg($row['NRORG']);
        $grupoIncidencia->setNrgrpincide(!empty($row['NRGRPINCIDE']) ? $row['NRGRPINCIDE'] : \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GRPINCIDE', 1, 12, true));
        $grupoIncidencia->setNmgrpincide($row['NMGRPINCIDE']);
        $grupoIncidencia->setDtinivigencia($row['DTINIVIGENCIA']);
        $grupoIncidencia->setDtfimvigencia($row['DTFIMVIGENCIA']);
        $grupoIncidencia->setDsgrpincide($row['DSGRPINCIDE']);
        $grupoIncidencia->setDtinclusao($row['DTINCLUSAO']);
        $grupoIncidencia->setCdoperinclusao($row['CDOPERINCLUSAO']);
        $grupoIncidencia->setNrorginclusao($row['NRORGINCLUSAO']);
        $grupoIncidencia->setDtultatu($row['DTULTATU']);
        $grupoIncidencia->setCdoperultatu($row['CDOPERULTATU']);
        $grupoIncidencia->setNrorgultatu($row['NRORGULTATU']);
        
        $this->entityManager->persist($grupoIncidencia);
        $this->entityManager->flush(); 
        
        return $grupoIncidencia->getNrgrpincide();
    }
    
    public function setItensGrupoIncidenciaPopUpParametroFolha($nrgrpincide, $inssterceiros, $inssempresa, $insssat, $inssparteempresaprolabore, $inssparteempresaautonomo, $nreventom, $liquidoguiagps){
        $ordem = 1;
        
        $dtinclusao = DateUtil::getDataAtual();
        $cdoperinclusao = $this->hcmParameters->getCdoperador();
        $nrorginclusao = $this->hcmParameters->getNrorgTrabalho();
        $dtultatu = $dtinclusao;
        $cdoperultatu = $cdoperinclusao;
        $nrorgultatu = $nrorginclusao;
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        if(!empty($inssterceiros) || !empty($inssempresa) || !empty($insssat) || !empty($inssparteempresaprolabore) || !empty($inssparteempresaautonomo) || !empty($liquidoguiagps)){
            $this->entityManager->beginTransaction();
            
            // % INSS Terceiros
            if(!empty($inssterceiros)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% INSS Terceiros', 3870, $nrgrpincide, null, $ordem++, $nrorg, 111, 1, 1, $inssterceiros, 0, 0, 9999999999);
            }
            // % INSS Empresa
            if(!empty($inssempresa)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% INSS Empresa', 3590, $nrgrpincide, null, $ordem++, $nrorg, 111, 1, 1, $inssempresa, 0, 0, 9999999999);
            }
            
            // % INSS SAT
            if(!empty($insssat)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% INSS SAT', 3600, $nrgrpincide, null, $ordem++, $nrorg, 111, 1, 1, $insssat, 0, 0, 9999999999);
            }
        
            // % INSS Parte Empresa Pro Labore
            if(!empty($inssparteempresaprolabore)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% INSS Parte Empresa Pro Labore', 3595, $nrgrpincide, null, $ordem++, $nrorg, 56, 1, 1, $inssparteempresaprolabore, 0, 0, 9999999999);
            }
            
            // % INSS Parte Empresa Autônomo
            if(!empty($inssparteempresaautonomo)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% INSS Parte Empresa Autônomo', $nreventom, $nrgrpincide, null, $ordem++, $nrorg, 145, 1, 1, $inssparteempresaautonomo, 0, 0, 9999999999);
            }
            
            // % Líquido Guia GPS
            if(!empty($liquidoguiagps)){
                $this->setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, null, 'PERCENTUAL', 'N', 'S', '% Líquido Guia GPS', 3580, $nrgrpincide, null, $ordem++, $nrorg, 71, 1, 1, $liquidoguiagps, 0, 0, 9999999999);
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function setFpaGrpItemDados($dtinclusao, $cdoperinclusao, $nrorginclusao, $dtultatu, $cdoperultatu, $nrorgultatu, $dsitem, $idrefgerar, $idretreferencia, $idretvalor, $nmitem, $nreventom, $nrgrpincide, $nrgrpitem, $nritem, $nrorg, $nrtipoincide, $vrdivide, $vrmultiplic, $vrpercent, $vrpisoevento, $vrreffixo, $vrtetoevento){
        $fpa_grpitem = Repositories::FPA_GRPITEM;
        $itemGrupoIncidencia = new $fpa_grpitem;
        
        if(empty($nrgrpitem)){
            $nrgrpitem = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GRPITEM', 1, 12, true);
        }
        
        $itemGrupoIncidencia->setDtinclusao($dtinclusao);
        $itemGrupoIncidencia->setCdoperinclusao($cdoperinclusao);
        $itemGrupoIncidencia->setNrorginclusao($nrorginclusao);
        $itemGrupoIncidencia->setDtultatu($dtultatu);
        $itemGrupoIncidencia->setCdoperultatu($cdoperultatu);
        $itemGrupoIncidencia->setNrorgultatu($nrorgultatu);
        $itemGrupoIncidencia->setDsitem($dsitem);
        $itemGrupoIncidencia->setIdrefgerar($idrefgerar);
        $itemGrupoIncidencia->setIdretreferencia($idretreferencia);
        $itemGrupoIncidencia->setIdretvalor($idretvalor);
        $itemGrupoIncidencia->setNmitem($nmitem);
        $itemGrupoIncidencia->setNreventom($nreventom);
        $itemGrupoIncidencia->setNrgrpincide($nrgrpincide);
        $itemGrupoIncidencia->setNrgrpitem($nrgrpitem);
        $itemGrupoIncidencia->setNritem($nritem);
        $itemGrupoIncidencia->setNrorg($nrorg);
        $itemGrupoIncidencia->setNrtipoincide($nrtipoincide);
        $itemGrupoIncidencia->setVrdivide($vrdivide);
        $itemGrupoIncidencia->setVrmultiplic($vrmultiplic);
        $itemGrupoIncidencia->setVrpercent($vrpercent);
        $itemGrupoIncidencia->setVrpisoevento($vrpisoevento);
        $itemGrupoIncidencia->setVrreffixo($vrreffixo);
        $itemGrupoIncidencia->setVrtetoevento($vrtetoevento);
        
        $this->entityManager->persist($itemGrupoIncidencia);
    }
}