<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 02/08/2016
 * Time: 18:15
 */

namespace HCM\FPA\Services;

use Folha\Entities\FpaItecalcfolha;
use Folha\Entities\FpaTipoassoceve;
use Folha\Entities\FpaTipomoviment;
use HCM\FPA\Repository\FpaitecalcfolhaRepository;
use HCM\FPA\Repository\BloqueioEventoVinculoRepository;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class ItensCalculoService {

    protected $hcmParameters;

    public function __construct(HCMParameters $hcmParameters, FpaitecalcfolhaRepository $fpaitecalcfolhaRepository, BloqueioEventoVinculoRepository $bloqueioVinculoRepository) {
        $this->hcmParameters = $hcmParameters;
        $this->fpaitecalcfolhaRepository = $fpaitecalcfolhaRepository;
        $this->FpaItecalcfolha = Repositories::FPA_ITECALCFOLHA;
        $this->bloqueioEventVincRepository = $bloqueioVinculoRepository;
    }

    /**
     * Verifica se o vínculo possui líquido no cálculo em questão.
     *
     * Em caso de movimento mensal, primeiro verifica se existe líquido de rescisão e retorna caso exista.
     * Caso não exista verifica líquido normal assim como para outros movimentos.
     *
     * @param $nrtipomoviment
     * @param $nrcalculofolha
     * @param $nrvinculom
     * @param $nrorg
     * @return bool
     */
    public function validaLiquido($nrtipomoviment, $nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg)
    {
        try {
            if ($nrtipomoviment == FpaTipomoviment::TP_MOV_MENSAL) {
                $liquido = FpaItecalcfolha::validaLiquido($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, FpaTipoassoceve::ASSOC_LIQUIDO_RESCISAO);
                if ($liquido) {
                    return $liquido;
                }
            }
            return FpaItecalcfolha::validaLiquido($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, FpaTipoassoceve::ASSOC_LIQUIDO);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaItemCalculo($nrorg, $nrcalculofolha, $nreventom, $nrvinculom, $dtocorrencia)
    {
        try {
            return $this->fpaitecalcfolhaRepository->getUniqueItem($nrorg, $nrcalculofolha, $nreventom, $nrvinculom, $dtocorrencia);
        } catch (\Exception $e) {
            throw $e;
        }
    }
   
    public function updateItensCalc($row){
        try {
            
            $iteCalc = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy(array('nrorg' => $row['NRORG'],
            'nrvinculom' => $row['NRVINCULOM'], 'nritemcalcfolha' => $row['NRITEMCALCFOLHA']));
            
            $idRefeValorCalc = $row['IDREFEVALORCALC'];
            
            if($row['IDREFEVALORCALC'] == 'NORMAL' || !$row['IDREFEVALORCALC']){
                if($row['VREVENTOFOLHA_inline'] && $row['VRREFFOLHA_inline']){
                    $idRefeValorCalc = 'REFERENCIA_VALOR';
                }else if($row['VREVENTOFOLHA_inline']){
                    $idRefeValorCalc = 'VALOR';
                }else if($row['VRREFFOLHA_inline']){
                    $idRefeValorCalc = 'REFERENCIA';
                }
            }
            
            $iteCalc->setVreventofolha($row['VREVENTOFOLHA']);
            $iteCalc->setVrreffolha($row['VRREFFOLHA']);
            $iteCalc->setIdrefevalorcalc($idRefeValorCalc);
            
            \Setforms2\Shared\Registry::getEntityManager()->persist($iteCalc);
            \Setforms2\Shared\Registry::getEntityManager()->flush($iteCalc);
            
        } catch (Exception $e) {
            throw $e;    
        }
    }
    
    public function verificaEventoBloqueado ($nrorg, $nrevento, $tipoVinculo) {
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrorg_padrao = $this->hcmParameters->getNrorgPadrao();
        return $this->bloqueioEventVincRepository->retornaEventosBloqueados($nrorg, $nrorg_padrao, $dtmescompetenc, $nrevento, $tipoVinculo);
    }
   
}