<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Services\AprovacaoAdmissaoService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\Util;

class AprovacaoAdmissaoController extends Crud {

    protected $parameterBagColumns = array();
    protected $dataSourceName = '';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AprovacaoAdmissaoService $aprovacaoAdmissaoService, ESocialService $eSocialService, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->aprovacaoAdmissaoService = $aprovacaoAdmissaoService;
        $this->eSocialService = $eSocialService; 
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }
    
    public function getVinculosNaoAprovados(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            set_time_limit(0); 
           $filters = $request->getFilterCriteria()->getConditions();
           $parameters = array();
           foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->aprovacaoAdmissaoService->getVinculosNaoAprovados($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculosNaoAprovados', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovaAdmissao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows()[0];
            foreach ($rows  as $row) {
                
                $row['NRESTRUTURAM'] = $row['NRESTRUTLEGAL'];
                if($row['NRTIPOVINCULOM'] == 1){
                    $row['NRCHAVE'] = $row['NRVINCULOM'];
                    $row['NMEVENTO'] = 'S-2200';
                }else{
                    $row['NRCHAVE'] = $row['NRVINCULOM'];
                    $row['NMEVENTO'] = 'S-2300';
                }
                
                $this->aprovacaoAdmissaoService->aprovaAdmissao($row);
                
                if(array_key_exists('IDENVIAESOCIAL', $rows[0]) && $rows[0]['IDENVIAESOCIAL'] == 'S'){
                    
                    $retornoEsocial2200 = [];
                    $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                    $dataSets = $response->getDataSets();
                        
                    foreach($dataSets as $dataSet){
                        if($dataSet->getDataSourceName() == 'retornoEsocial'){
                            $retornoEsocial2200[0] = ['VINCULO'  => $row['NRVINCULOM'],
                                                      'RETORNO'  => $dataSet->getRows()[0]];
                        }
                    }
                    $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial2200', $retornoEsocial2200)));
                }
            }
            
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('aprova', array(true)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setAlteracoesAprova(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach ($request->getDataSet()->getRows()  as $row) {
                $this->aprovacaoAdmissaoService->setAlteracoesAprova($row);
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('aprova', array(true)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
}