<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\Messages;
use Doctrine\ORM\EntityManager;

class FornecedorController extends Crud {
    
    
    protected $dataSourceName = 'getFornecedor';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    protected $fornecedorService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\Geral\Services\FornecedorService $fornecedorService, EntityManager $entityManager){
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->fornecedorService = $fornecedorService;
        parent::__construct($dataSourceManager);
    }

    public function getFornecedor (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setFornecedor (DTO\Request\DataSet $request, DTO\Response $response) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorg_padrao = $this->hcmParameters->getNrorgPadrao();
        foreach($request->getDataSet()->getRows() as &$row) {
            $cdfornecedor = $this->entityManager->getRepository(Repositories::FORNECEDOR)->findBy(array('nrorg' => array($nrorg, $nrorg_padrao), 'cdforneced' => $row['CDFORNECED']));
            $remove_mask = array("." => "","-" => "","/" => "");
            $row['NRINSJURFORN'] = $row['NRINSJURFORN'] = strtr($row['NRINSJURFORN'],$remove_mask);
            $inscricaofornecedor = $this->entityManager->getRepository(Repositories::FORNECEDOR)->findBy(array('nrorg' => array($nrorg, $nrorg_padrao), 'nrinsjurforn' => $row['NRINSJURFORN']));
            
            if ($row['__is_new']) {
                if(empty($cdfornecedor) && empty($inscricaofornecedor)){
                    try {
                      $this->fornecedorService->salvarAdicao($row);
                      $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("success"));
                    }catch(\Exception $e) {
                      $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
                    }
                } else if (!empty($cdfornecedor)) {
                    $response->addMessage(new DTO\Response\Message("Já existe um fornecedor cadastrado com esse código", DTO\Response\Message::TYPE_ERROR));
                } else {
                    $response->addMessage(new DTO\Response\Message("Já existe um fornecedor cadastrado com essa inscrição", DTO\Response\Message::TYPE_ERROR));
                }
            } else {
                try {
                  $this->fornecedorService->salvarEdicao($row);
                  $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("success"));
                }catch(\Exception $e) {
                  $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
                }
            }
        }

        return true;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}