<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;

class AlteSituacaoFuncionalRepository {

    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }

    public function retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc) {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $sql = <<<SQL
SELECT a
  FROM $gpeAltesitufunc a
WHERE a.nrorg         = :NRORG
  AND a.nrvinculom    = :NRVINCULOM
  AND a.dtinisitufunc = (SELECT MAX(b.dtinisitufunc)
                           FROM $gpeAltesitufunc b
                          WHERE b.nrorg       = a.nrorg
                            AND b.nrvinculom  = a.nrvinculom
                            AND b.dtinisitufunc <= :DTINISITUFUNC)
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'NRVINCULOM' => $nrvinculom,
            'DTINISITUFUNC' => $dtinisitufunc
        );
        $result = $this->entityManager->createQuery($sql)->execute($params);
        return empty($result) ? $result : $result[0];
    }

    public function retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc=null, $nraltesitufunc = null) {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $sql = <<<SQL
SELECT a
  FROM $gpeAltesitufunc a
WHERE a.nrorg         = :NRORG
  AND a.nrvinculom    = :NRVINCULOM
  AND a.dtinisitufunc > :DTINISITUFUNC
  AND (:NRALTESITUFUNC IS NULL OR a.nraltesitufunc <> :NRALTESITUFUNC)
  AND (:NRSITUFUNCM IS NULL OR a.nrsitufuncm = :NRSITUFUNCM)
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'NRVINCULOM' => $nrvinculom,
            'NRSITUFUNCM' => $nrproxsitufunc,
            'DTINISITUFUNC' => $dtinisitufunc,
            'NRALTESITUFUNC' => $nraltesitufunc,
        );

        $result = $this->entityManager->createQuery($sql)->execute($params);

        return empty($result) ? $result : $result[0];
    }

    public function retornaAlteSitFuncporVinculoNrcid($nr_vinculo,$nr_situacao,$nr_cid,$nr_org)
    {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $query = <<<SQL
SELECT a
FROM $gpeAltesitufunc a
WHERE a.nrvinculom = :P_NRVINCULOM
AND a.nrsitufuncm = :P_NRSITUFUNCM
AND a.cdtabecdi = :P_CDTABECDI
AND a.nrorg = :P_NRORG
SQL;
        $params = array(
            'P_NRVINCULOM'  => $nr_vinculo,
            'P_NRSITUFUNCM' => $nr_situacao,
            'P_CDTABECDI'   => $nr_cid,
            'P_NRORG'       => $nr_org,
        );

        return $this->entityManager->createQuery($query)->execute($params);
    }

} 
