<?php
namespace HCM\PTO\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class ResumosGeraisApuracaoRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    /**
     * @return array
     */
    public function getResumosGeraisApuracaoDiario($dataset) {
        $nrorg = $dataset['NRORGTRAB'];
        $nrperiodoapuracao = $dataset['NRPERIODOAPURACAO'];
        $dtiniperiodoapuracao = explode(' ', $dataset['DTINICIALAPURACAO'])[0];
        $dtfimperiodoapuracao = explode(' ', $dataset['DTFINALAPURACAO'])[0];
        $filter = '';
        if(array_key_exists('NRVINCULOM_LIST', $dataset) && !empty($dataset['NRVINCULOM_LIST'])){
            $nrvinculom = implode(",", $dataset['NRVINCULOM_LIST']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOM.NRVINCULOM IN ($nrvinculom)";
            } else {
                $filter .= " AND GPE_VINCULOM.NRVINCULOM IN ($nrvinculom)";
            }
        }

        if(array_key_exists('LIST_NRESTRUTLEGAL', $dataset) && !empty($dataset['LIST_NRESTRUTLEGAL'])){
            $nrestruturalegal = implode(",", $dataset['LIST_NRESTRUTLEGAL']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOH.NRESTRUTLEGAL IN ($nrestruturalegal)";
            } else {
                $filter .= " AND GPE_VINCULOH.NRESTRUTLEGAL IN ($nrestruturalegal)";
            }
        }

        if(array_key_exists('LIST_NRESTRUTGEREN', $dataset) && !empty($dataset['LIST_NRESTRUTGEREN'])){
            $nrestruturagerencial = implode(",", $dataset['LIST_NRESTRUTGEREN']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOH.NRESTRUTGEREN IN ($nrestruturagerencial)";
            } else {
                $filter .= " AND GPE_VINCULOH.NRESTRUTGEREN IN ($nrestruturagerencial)";
            }
        }
        
        $query = "
                SELECT
                    *
                  FROM (SELECT
                          TO_CHAR (VINCULOS.DTDATA, 'YYYY/MM/DD') DATE_ORDER,
                          TO_CHAR (VINCULOS.DTDATA, 'DD') DIA_MES,
                          TO_CHAR (VINCULOS.DTDATA, 'MM') MES,
                          REPLACE(TO_CHAR (VINCULOS.DTDATA, 'DD/MM/YYYY') || ' - ' || INITCAP(VINCULOS.DIA), '-Feira') DTAPURACAOSITUFREQ,
                          PRINCIPAL.TRAB_DIURNO_I,
                          PRINCIPAL.SALDO_A_FOLGAR_NORMAL,
                          PRINCIPAL.SALDO_FOLGAR_PER_BHR_NORM,
                          PRINCIPAL.SALDO_FOLGAR_ACR_PER_BHR_NORM,
                          PRINCIPAL.EX_DIA_NORMAL_SEG,
                          PRINCIPAL.EX_DIA_NORMAL_NP_SEG,
                          PRINCIPAL.ATRASOS_DIURNOS,
                          PRINCIPAL.FALTAS_DIURNAS,
                          VINCULOS.NMVINCULOM,
                          VINCULOS.NRVINCULOM,
                          PRINCIPAL.NRPERIODOAPURACAO
                        FROM
                        (SELECT
                          DTAPURACAOSITUFREQ DATA_APURACAO,
                          TO_CHAR (DTAPURACAOSITUFREQ, 'DD') DIA_MES,
                          TO_CHAR (DTAPURACAOSITUFREQ, 'MM') MES,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(TRAB_DIURNO_I), '-') TRAB_DIURNO_I,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(SALDO_A_FOLGAR_NORMAL), '-') SALDO_A_FOLGAR_NORMAL,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(SALDO_FOLGAR_PER_BHR_NORM), '-') SALDO_FOLGAR_PER_BHR_NORM,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(SALDO_FOLGAR_ACR_PER_BHR_NORM), '-') SALDO_FOLGAR_ACR_PER_BHR_NORM,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(EX_DIA_NORMAL_SEG), '-') EX_DIA_NORMAL_SEG,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(EX_DIA_NORMAL_NP_SEG), '-') EX_DIA_NORMAL_NP_SEG,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(ATRASOS_DIURNOS), '-') ATRASOS_DIURNOS,
                          DECODE(IS_FERIADO, NULL, TO_CHAR(FALTAS_DIURNAS), '-') FALTAS_DIURNAS,
                          NMVINCULOM,
                          NRVINCULOM,
                          NRPERIODOAPURACAO
                        FROM (
                            SELECT * FROM (
                                SELECT DTAPURACAOSITUFREQ, NRSITUFREQUENCIA, (ROUND(QTREFAPURSITUFREQ, 2)) QTREFAPURSITUFREQ,
                                       (SELECT DTCALENDARIO
                                          FROM OCORCALENDARIO
                                         WHERE NRCALENDARIO = 1
                                           AND NRORG = :P_NRORG
                                           AND DTCALENDARIO = DTAPURACAOSITUFREQ) IS_FERIADO,
                                       VM.NMVINCULOM,
                                       VM.NRVINCULOM,
                                       NRPERIODOAPURACAO
                                  FROM PTO_RESUMOSITUFREQ
                                  JOIN GPE_VINCULOM VM
                                    ON VM.NRVINCULOM = PTO_RESUMOSITUFREQ.NRVINCULOM
                                   AND VM.NRORG      = PTO_RESUMOSITUFREQ.NRORG
                                 WHERE PTO_RESUMOSITUFREQ.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                                   AND PTO_RESUMOSITUFREQ.NRORG             = :P_NRORG
                            ) PTO_RESUMOSITUFREQ
                        ) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (100 TRAB_DIURNO_I, 18000 SALDO_A_FOLGAR_NORMAL, 18020 SALDO_FOLGAR_PER_BHR_NORM, 18420 SALDO_FOLGAR_ACR_PER_BHR_NORM, 2100 EX_DIA_NORMAL_NP_SEG, 2300 EX_DIA_NORMAL_SEG, 10100 ATRASOS_DIURNOS, 10500 FALTAS_DIURNAS))

                        ORDER BY MES, DIA_MES) PRINCIPAL
                        RIGHT JOIN (SELECT
                                      DTDATA,
                                      GPE_VINCULOM.NRORG,
                                      GPE_VINCULOM.NRVINCULOM,
                                      GPE_VINCULOM.NMVINCULOM,
                                       TO_CHAR(DTDATA, 'DAY', 'NLS_DATE_LANGUAGE = PORTUGUESE') AS DIA
                                FROM DATA, GPE_VINCULOM
                                INNER JOIN GPE_VINCULOH
                                    ON GPE_VINCULOH.NRVINCULOM      = GPE_VINCULOM.NRVINCULOM
                                    AND GPE_VINCULOH.NRORG          = GPE_VINCULOM.NRORG
                                    AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(VH.DTMESCOMPETENC)
                                                                      FROM GPE_VINCULOH VH
                                                                      WHERE VH.NRORG      = GPE_VINCULOH.NRORG
                                                                      AND VH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM)
                                WHERE DTDATA BETWEEN :P_DTINIPERIODOAPURACAO AND :P_DTFIMPERIODOAPURACAO
                                  AND GPE_VINCULOM.NRORG = :P_NRORG
                                  ".$filter."
                                  AND EXISTS (SELECT (1)
                                              FROM PTO_RESUMOSITUFREQ
                                             WHERE NRORG = GPE_VINCULOM.NRORG
                                               AND NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                                               AND QTREFAPURSITUFREQ != '0' /* verificar se pode ser validado dessa forma */
                                               AND DTAPURACAOSITUFREQ BETWEEN TO_DATE(:P_DTINIPERIODOAPURACAO) AND TO_DATE(:P_DTFIMPERIODOAPURACAO) -1)
                                ORDER BY NMVINCULOM, DTDATA) VINCULOS
                          ON VINCULOS.NRVINCULOM = PRINCIPAL.NRVINCULOM
                         AND VINCULOS.DTDATA     = PRINCIPAL.DATA_APURACAO
                         WHERE 1 = 1 ORDER BY VINCULOS.NRVINCULOM DESC) PRINCIPAL
                  WHERE 1 = 1 ORDER BY PRINCIPAL.DATE_ORDER";
        $params = array('P_NRORG'                => $nrorg,
                        'P_NRPERIODOAPURACAO'    => $nrperiodoapuracao,
                        'P_DTINIPERIODOAPURACAO' => $dtiniperiodoapuracao,
                        'P_DTFIMPERIODOAPURACAO' => $dtfimperiodoapuracao);
           
        return $this->connection->fetchAll($query, $params);
    }

    public function getResumosGeraisApuracaoPeriodo($dataset) {
        $nrorg = $dataset['NRORGTRAB'];
        $nrperiodoapuracao = $dataset['NRPERIODOAPURACAO'];
        $nrtipovinculom = $dataset['NRTIPOVINCULOM'];
        $filter = '';
        if(array_key_exists('NRVINCULOM_LIST', $dataset) && !empty($dataset['NRVINCULOM_LIST'])){
            $nrvinculom = implode(",", $dataset['NRVINCULOM_LIST']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOM.NRVINCULOM IN ($nrvinculom)";
            } else {
                $filter .= " AND GPE_VINCULOM.NRVINCULOM IN ($nrvinculom)";
            }
        }

        if(array_key_exists('LIST_NRESTRUTLEGAL', $dataset) && !empty($dataset['LIST_NRESTRUTLEGAL'])){
            $nrestruturalegal = implode(",", $dataset['LIST_NRESTRUTLEGAL']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOH.NRESTRUTLEGAL IN ($nrestruturalegal)";
            } else {
                $filter .= " AND GPE_VINCULOH.NRESTRUTLEGAL IN ($nrestruturalegal)";
            }
        }

        if(array_key_exists('LIST_NRESTRUTGEREN', $dataset) && !empty($dataset['LIST_NRESTRUTGEREN'])){
            $nrestruturagerencial = implode(",", $dataset['LIST_NRESTRUTGEREN']);
            if(empty($filter)){
                $filter = "AND GPE_VINCULOH.NRESTRUTGEREN IN ($nrestruturagerencial)";
            } else {
                $filter .= " AND GPE_VINCULOH.NRESTRUTGEREN IN ($nrestruturagerencial)";
            }
        }
        $query = "
                  SELECT NRVINCULOM,
                         NMVINCULOM,
                         NRPERIODOAPURACAO,
                         PTO_RESUMOSITUFREQ.NRESTRUTLEGAL,
                         PTO_RESUMOSITUFREQ.NMLEGAL NRESTRUTLEGAL__LABEL,
                         PTO_RESUMOSITUFREQ.NRESTRUTGEREN,
                         PTO_RESUMOSITUFREQ.NMGERENCIAL NRESTRUTGEREN__LABEL,
                         DECODE(PTO_RESUMOSITUFREQ.SALDO_FOLGAR_PER_BHR_NORM,     NULL, '0000', PTO_RESUMOSITUFREQ.SALDO_FOLGAR_PER_BHR_NORM) SALDO_FOLGAR_PER_BHR_NORM,
                         DECODE(PTO_RESUMOSITUFREQ.SALDO_FOLGAR_ACR_PER_BHR_NORM, NULL, '0000', PTO_RESUMOSITUFREQ.SALDO_FOLGAR_ACR_PER_BHR_NORM) SALDO_FOLGAR_ACR_PER_BHR_NORM,
             DECODE(PTO_RESUMOSITUFREQ.SALDO_FOLGAR_ACR_PER_BHR_NORM, NULL, '0000', PTO_RESUMOSITUFREQ.SALDO_FOLGAR_ACR_PER_BHR_NORM * 0.8) SALDO_FOLGAR_ACR_PER_BHR_25,
                         DECODE(PTO_RESUMOSITUFREQ.DSR_DESCONTADO,                NULL, '0000', PTO_RESUMOSITUFREQ.DSR_DESCONTADO) DSR_DESCONTADO,
                         DECODE(PTO_RESUMOSITUFREQ.FALTAS_DIURNAS,                NULL, '0000', PTO_RESUMOSITUFREQ.FALTAS_DIURNAS) FALTAS_DIURNAS,
                         DECODE(PTO_RESUMOSITUFREQ.EX_DIA_NORMAL_NP_SEG,          NULL, '0000', PTO_RESUMOSITUFREQ.EX_DIA_NORMAL_NP_SEG) EX_DIA_NORMAL_NP_SEG,
                         DECODE(PTO_RESUMOSITUFREQ.EX_DIA_NORMAL_SEG,             NULL, '0000', PTO_RESUMOSITUFREQ.EX_DIA_NORMAL_SEG) EX_DIA_NORMAL_SEG,
                         DECODE(PTO_RESUMOSITUFREQ.ATRASOS_DIURNOS,               NULL, '0000', PTO_RESUMOSITUFREQ.ATRASOS_DIURNOS) ATRASOS_DIURNOS,
                         DECODE(PTO_RESUMOSITUFREQ.ADICIONAL_NOTURNO,             NULL, '0000', PTO_RESUMOSITUFREQ.ADICIONAL_NOTURNO) ADICIONAL_NOTURNO
                  FROM (SELECT GPE_VINCULOM.NRVINCULOM,
                               NMVINCULOM AS NMVINCULOM,
                               NRSITUFREQUENCIA,
                               NVL(ROUND(QTREFAPURSITUFREQ, 7), '0000') QTREFAPURSITUFREQ,
                               RESUMOS.NRPERIODOAPURACAO,
                               GPE_VINCULOH.NRESTRUTLEGAL,
                               GPE_VINCULOH.NRESTRUTGEREN,
                               LEGAL.NMESTRUTURAM NMLEGAL,
                               GERENCIAL.NMESTRUTURAM NMGERENCIAL,
                              (SELECT NVL(ROUND(QTREFAPURSITUFREQ, 7), '0000')
                                 FROM (SELECT QTREFAPURSITUFREQ
                                         FROM PTO_RESUMOSITUFREQ
                                        WHERE NRVINCULOM        = RESUMOS.NRVINCULOM
                                          AND NRPERIODOAPURACAO = $nrperiodoapuracao
                                          AND NRORG             = $nrorg
                                          AND NRSITUFREQUENCIA  = 18020
                                     ORDER BY DTAPURACAOSITUFREQ DESC)
                                WHERE ROWNUM = 1) AS SALDO_FOLGAR_PER_BHR_NORM,
                              (SELECT NVL(ROUND(QTREFAPURSITUFREQ, 7), '0000')
                                 FROM (SELECT QTREFAPURSITUFREQ
                                         FROM PTO_RESUMOSITUFREQ
                                        WHERE NRVINCULOM        = RESUMOS.NRVINCULOM
                                          AND NRPERIODOAPURACAO = $nrperiodoapuracao
                                          AND NRORG             = $nrorg
                                          AND NRSITUFREQUENCIA  = 18420
                                     ORDER BY DTAPURACAOSITUFREQ DESC)
                                WHERE ROWNUM = 1) AS SALDO_FOLGAR_ACR_PER_BHR_NORM,
                              (SELECT COUNT(QTREFAPURSITUFREQ)
                                 FROM PTO_RESUMOSITUFREQ
                                WHERE NRVINCULOM        = RESUMOS.NRVINCULOM
                                  AND NRPERIODOAPURACAO = $nrperiodoapuracao
                                  AND NRORG             = $nrorg
                                  AND NRSITUFREQUENCIA  = 12100) AS DSR_DESCONTADO,
                              (SELECT COUNT(QTREFAPURSITUFREQ)
                                 FROM PTO_RESUMOSITUFREQ
                                WHERE NRVINCULOM        = RESUMOS.NRVINCULOM
                                  AND NRPERIODOAPURACAO = $nrperiodoapuracao
                                  AND NRORG             = $nrorg
                                  AND NRSITUFREQUENCIA  = 10500) AS FALTAS_DIURNAS
                          FROM PTO_RESUMOSITUFREQ RESUMOS
                          JOIN GPE_VINCULOM
                            ON GPE_VINCULOM.NRVINCULOM = RESUMOS.NRVINCULOM
                           AND GPE_VINCULOM.NRORG      = $nrorg
                          JOIN GPE_VINCULOH
                            ON GPE_VINCULOH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                           AND GPE_VINCULOH.NRORG      = GPE_VINCULOM.NRORG
                           AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(VH.DTMESCOMPETENC)
                                                                FROM GPE_VINCULOH VH
                                                               WHERE VH.NRORG      = GPE_VINCULOH.NRORG
                                                                 AND VH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM)
                          JOIN  GPE_TIPOVINCULOM
                            ON (GPE_TIPOVINCULOM.NRORG              = RESUMOS.NRORG OR
                                GPE_TIPOVINCULOM.NRORG              = 0)
                           AND GPE_TIPOVINCULOM.NRTIPOVINCULOM      = GPE_VINCULOM.NRTIPOVINCULOM
                          JOIN ESTRUTURAM LEGAL
                            ON LEGAL.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTLEGAL
                           AND LEGAL.NRORG        = GPE_VINCULOH.NRORG
                          JOIN ESTRUTURAM GERENCIAL
                            ON GERENCIAL.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTGEREN
                           AND GERENCIAL.NRORG        = GPE_VINCULOH.NRORG
                         WHERE RESUMOS.NRPERIODOAPURACAO = $nrperiodoapuracao
                           AND RESUMOS.NRORG             = $nrorg
                           AND GPE_TIPOVINCULOM.NRTIPOVINCULOM = $nrtipovinculom
                           $filter
                  ) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (2100 EX_DIA_NORMAL_NP_SEG, 2300 EX_DIA_NORMAL_SEG, 10100 ATRASOS_DIURNOS, 9000 ADICIONAL_NOTURNO)) PTO_RESUMOSITUFREQ ORDER BY PTO_RESUMOSITUFREQ.NRVINCULOM";
        
        return $this->connection->fetchAll($query);
    }
    
    public function getCdoperadorResponsavel($nrpessoa, $nrorg){
        
        $params = array('P_NRPESSOA' => $nrpessoa,
                        'P_NRORG'    => $nrorg);
        $query = "SELECT CDOPERADOR 
                    FROM OPERADOR 
               LEFT JOIN GPE_PESSOA
                      ON OPERADOR.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                     AND OPERADOR.NRORG = GPE_PESSOA.NRORG
                   WHERE GPE_PESSOA.NRPESSOA = :P_NRPESSOA
                     AND GPE_PESSOA.NRORG = :P_NRORG";
        
        return $this->connection->fetchAll($query, $params);
        
    }
    
    public function getResumosGeraisApuracaoResponsavel($dataset){
        
        $params = array('P_NRORG'             => $dataset['NRORGTRAB'],
                        'P_NRPERIODOAPURACAO' => $dataset['NRPERIODOAPURACAO'],
                        'P_DTFINALAPURACAO'   => substr($dataset['DTFINALAPURACAO'], 0, 10),
						'P_NRESTRUTLEGAL'     => $dataset['NRESTRUTLEGAL'],
						'P_NRESTRUTGEREN'     => $dataset['NRESTRUTGEREN'],
						'P_NRESTRUTSIND'      => $dataset['NRESTRUTSIND'],
						'P_NRVINCULOM'        => $dataset['NRVINCULOM'],
						'P_OPERADORLIDER'     => $dataset['CDOPERADORRESP'] );
        
        $query = "SELECT PRINCIPAL.*,
                           ESTRUTURAM.NMESTRUTURAM NMESTRUTGEREN,
                    	   PARCNEGOCIO.NMPRINCIPALPARC NMPESSOA	
                      FROM TABLE(BANCOHORAS_DADOS(:P_NRORG, :P_NRPERIODOAPURACAO, :P_DTFINALAPURACAO, :P_NRESTRUTLEGAL, :P_NRESTRUTGEREN, :P_NRESTRUTSIND, :P_NRVINCULOM, :P_OPERADORLIDER)) PRINCIPAL,
                    	   ESTRUTURAM,
                    	   PARCNEGOCIO
                     WHERE ESTRUTURAM.NRORG = :P_NRORG	   
                       AND ESTRUTURAM.NRESTRUTURAM = PRINCIPAL.NRESTRUTGEREN
                       AND PARCNEGOCIO.NRORG = :P_NRORG
                       AND PARCNEGOCIO.NRPARCNEGOCIO = PRINCIPAL.NRPARCLIDER";
	    
    
        return $this->connection->fetchAll($query, $params);
    }
} 


        